package widebase.rq.handler

import msgpack.rpcx.Server
import msgpack.rpcx.auth. { AuthMap, LoginMap }

import scala.collection.mutable.ArrayBuffer
import scala.concurrent.Lock

import vario.filter.StreamFilter
import vario.filter.StreamFilter.StreamFilter

import widebase.rq.concurrent.Consumer
import widebase.rq.persistence.PersistenceWriter

/** Collect broker handler.
 *
 * @param path to persistence
 * @param server instance
 * @param jaas entry name
 * @param auths procedure based authorizations
 * @param interval till client timeout
 * @param filter self-explanatory
 *
 * @author myst3r10n
 */
class BrokerHandler (
  val path: String,
  val server: Server,
  val jaas: String,
  val auths: AuthMap,
  val interval: Int = 0,
  val filter: StreamFilter = StreamFilter.None)
  extends ControlHandler
  with PublishingHandler
  with SubscribingHandler {

  /** Locks sensitive data. */
  protected val lock = new Lock

  /** Log instance. */
  protected def log = widebase.rq.handler.ref

  /** Holds authorizations. */
  val loginMap = new LoginMap

  /** Release resources. */
  def release {

    subscriptions.clear
    persistences.clear
    consumers.values.foreach(consumer => consumer.shutdown)
    consumers.clear

  }
}

