package widebase.rq.handler

import java.io. { PrintWriter, StringWriter }

import msgpack.rpcx.Server
import msgpack.rpcx.handler.AuthHandler

import org.msgpack.rpc.Request

/** Handles server controls.
 *
 * @author myst3r10n
 */
trait ControlHandler extends AuthHandler {

  protected val server: Server

  /** Receives server shutdown request from client.
   *
   * @param request of client
   * @param authId unique authentication id of client
   */
  def remoteShutdown(request: Request, authId: String) {

    try {

      isLoggedIn(authId)
      hasAuthorization(authId, "remoteShutdown")

      log.info("Remote shutdown called: " + loginMap(authId).username + " (" +
        authId + ")")
      request.sendResult(true)

      server.release

    } catch {

      case e: Throwable =>
        val message = new StringWriter
        val printer = new PrintWriter(message)
        e.printStackTrace(printer)
        log.error(message.toString)
        request.sendError(e.toString)

    }
  }
}

