package widebase.rq.handler

import java.io. { PrintWriter, StringWriter }

import msgpack.rpcx.handler.AuthHandler

import org.msgpack.rpc.Request

import scala.collection.mutable. { ArrayBuffer, HashMap }
import scala.concurrent.Lock

import widebase.db.table.Table

/** Handler to processing producers.
 *
 * @author myst3r10n
 */
trait PublishingHandler extends PersistenceHandler {

  /** Locks sensitive data. */
  protected val lock: Lock

  /** Receives upsert request from producer.
   *
   * @param request of consumer
   * @param authId unique authentication id of consumer
   * @param name of table
   * @param bytes of records
   */
  protected def upsert(
    request: Request,
    authId: String,
    name: String,
    bytes: Array[Byte]) {

    lock.acquire

    try {

      isLoggedIn(authId)
      hasAuthorization(authId, "upsert")

      if(!path.isEmpty)
        write(name, bytes)

      if(subscriptions.contains(name))
        subscriptions(name).foreach(consumer => consumer.upsert(bytes))

      request.sendResult(true)

    } catch {

      case e: Throwable =>
        val message = new StringWriter
        val printer = new PrintWriter(message)
        e.printStackTrace(printer)
        log.error(message.toString)
        request.sendError(e.toString)

        return

    } finally {

      lock.release

    }
  }
}

